`timescale 1ns / 1ns

module codec_interface(RST, CLK,
                       RNW, EN, AIN, BE, DIN,
                       DOUT, ACK, INTR,

                       BICK, LRCK,
                       SDTI1, SDTI2, SDTI3, SDTI4,
                       SDTO, PDN);
   
   input RST;
   input CLK;

   wire  RST;
   wire  CLK;

   input RNW;
   input EN;
   input [0:7] AIN;
   input [0:31] DIN; 
   input [0:3] BE;
   
   wire         RNW;
   wire         EN;
   wire [0:7]   AIN;
   wire [0:31]  DIN;
   wire [0:3] BE;
   wire all_lanes;
   assign all_lanes = BE == 4'b1111;
   
   output [0:31] DOUT;
   output        ACK;
   output        INTR;

   reg [0:31]    DOUT, next_DOUT;
   reg           ACK, next_ACK, prev_ACK;
   reg           INTR, next_INTR;



   input         BICK;
   input         LRCK;
   
   wire          BICK; 
   wire          LRCK; 
   
   reg           prev_LRCK;
   wire          rising_LRCK;
   assign        rising_LRCK = LRCK && ~prev_LRCK;

   output        SDTI1;
   output        SDTI2;
   output        SDTI3;
   output        SDTI4;
   input         SDTO;
   output        PDN;
   
   wire          SDTI1; 
   wire          SDTI2; 
   wire          SDTI3; 
   wire          SDTI4;
   wire          SDTO;
   wire          PDN;
   

      
   reg [0:31]    control_register, next_control_register;
   reg [0:47]    input_stream, next_input_stream;
   wire [0:47]   input_data;
   reg [0:47]    stream1, next_stream1;
   reg [0:47]    stream2, next_stream2;
   reg [0:47]    stream3, next_stream3;
   reg [0:47]    stream4, next_stream4;
   
   


   wire          interrupt_enable;
   assign        interrupt_enable = control_register[30];
   assign        PDN = control_register[31];


   always @(posedge CLK)
     begin
        prev_LRCK = LRCK;
        if(RST)
          begin
             prev_ACK <= 0;
             DOUT <= 0;
             ACK <= 0;
             INTR <= 0;
             
             control_register <= 0;
             input_stream <= 0;
             stream1 <= 0;
             stream2 <= 0;
             stream3 <= 0;
             stream4 <= 0;
          end // if (RST)
        else
          begin
             DOUT <= EN && next_ACK && RNW ? next_DOUT : 0;
             prev_ACK <= ACK;
             ACK <= EN ? next_ACK : 0;
             INTR <= next_INTR;

             control_register <= next_control_register;
             input_stream <= next_input_stream;
             stream1 <= next_stream1;
             stream2 <= next_stream2;
             stream3 <= next_stream3;
             stream4 <= next_stream4;
          end // else: !if(RST)
     end // always @ (posedge CLK)

   always @(*)
     begin
        next_INTR = INTR;
        if(rising_LRCK && interrupt_enable) next_INTR = 1;
        if(EN) next_INTR = 0;
     end

   always @(*)
     begin
        next_input_stream = input_stream;
        if(rising_LRCK) next_input_stream = input_data;
     end

   always @(*)
     begin
        next_DOUT = 0;
        next_ACK = 0;
        next_control_register = control_register;
        next_stream1 = stream1;
        next_stream2 = stream2;
        next_stream3 = stream3;
        next_stream4 = stream4;

        if(EN)
          begin
             next_DOUT = {20'h0bad0, 2'b00,  EN, RNW ,AIN};
             if(!ACK && !prev_ACK) next_ACK = 1;
             case({RNW, AIN})
               9'h100: next_DOUT = control_register;
               9'h104: next_DOUT = {{8{input_stream[ 0]}}, input_stream[ 0:23]};
               9'h108: next_DOUT = {{8{input_stream[24]}}, input_stream[24:47]};
               9'h10C: next_DOUT = {{8{stream1[ 0]}}, stream1[ 0:23]};
               9'h110: next_DOUT = {{8{stream1[24]}}, stream1[24:47]};
               9'h114: next_DOUT = {{8{stream2[ 0]}}, stream2[ 0:23]};
               9'h118: next_DOUT = {{8{stream2[24]}}, stream2[24:47]};
               9'h11C: next_DOUT = {{8{stream3[ 0]}}, stream3[ 0:23]};
               9'h120: next_DOUT = {{8{stream3[24]}}, stream3[24:47]};
               9'h124: next_DOUT = {{8{stream4[ 0]}}, stream4[ 0:23]};
               9'h128: next_DOUT = {{8{stream4[24]}}, stream4[24:47]};
               9'h000: next_control_register[0:31] = DIN[0:31];
               9'h00C: next_stream1[ 0:23] = DIN[8:31];
               9'h010: next_stream1[24:47] = DIN[8:31];
               9'h014: next_stream2[ 0:23] = DIN[8:31];
               9'h018: next_stream2[24:47] = DIN[8:31];
               9'h01C: next_stream3[ 0:23] = DIN[8:31];
               9'h020: next_stream3[24:47] = DIN[8:31];
               9'h024: next_stream4[ 0:23] = DIN[8:31];
               9'h028: next_stream4[24:47] = DIN[8:31];
             endcase // case(AIN) 
			 if(!all_lanes) next_DOUT = 32'hbaadf00d;
          end // if (EN)
     end // always @ (*)

   codec_transmitter c_transmitter1
     (
      .RST(RST),
      .CLK(CLK),
      .BICK(BICK),
      .LRCK(LRCK),
      .SDOUT(SDTI1),
      .DATA(stream1)
      );

   codec_transmitter c_transmitter2
     (
      .RST(RST),
      .CLK(CLK),
      .BICK(BICK),
      .LRCK(LRCK),
      .SDOUT(SDTI2),
      .DATA(stream2)
      );

   codec_transmitter c_transmitter3
     (
      .RST(RST),
      .CLK(CLK),
      .BICK(BICK),
      .LRCK(LRCK),
      .SDOUT(SDTI3),
      .DATA(stream3)
      );
   

   codec_transmitter c_transmitter4
     (
      .RST(RST),
      .CLK(CLK),
      .BICK(BICK),
      .LRCK(LRCK),
      .SDOUT(SDTI4),
      .DATA(stream4)
      );

   codec_receiver c_receiver
     (
      .RST(RST),
      .CLK(CLK),
      .BICK(BICK),
      .LRCK(LRCK),
      .SDIN(SDTO),
      .DATA(input_data)
      );
   
endmodule // codec_interface

        
          