`timescale 1ns / 1ns

// note that DATA is only valid on the rising edge of lrck and it is
// left channel: bits 0:23 and right channel 24:47
module codec_transmitter(RST, CLK,
                         BICK, LRCK,
                         SDOUT, DATA);

   parameter WAIT_FOR_RISING_LRCK   = 0,
             WRITING_LEFT           = 1,
             WAIT_FOR_FALLING_LRCK  = 2,
             WRITING_RIGHT          = 3;
   
             
   input RST;
   input CLK;
   input BICK;
   input LRCK;
   input [0:47] DATA;
   
   wire  RST;
   wire  CLK;
   wire  BICK;
   wire  LRCK;
   wire [0:47] DATA;
   
   output      SDOUT;
   wire        SDOUT;
   reg [0:47]  shift_reg, next_shift_reg;
   
   reg   prev_bick, prev_lrck;
   wire  falling_bick, falling_lrck, rising_lrck;
   assign falling_bick = ~BICK && prev_bick;
   assign falling_lrck = ~LRCK && prev_lrck;
   assign rising_lrck = LRCK && ~prev_lrck;
   
   reg [0:5] counter, next_counter;
   reg [0:1] state, next_state;

   assign      SDOUT = (state == WRITING_LEFT || state == WRITING_RIGHT) ? shift_reg[0] : 0;
   
   always @(posedge CLK)
     begin
        prev_bick <= BICK;
        prev_lrck <= LRCK;
        
        if(RST)
          begin
             counter <= 0;
             state <= 0;
             shift_reg <= 0;
          end
        else
          begin
             counter <= next_counter;
             state <= next_state;
             shift_reg <= next_shift_reg;
          end // else: !if(RST)
     end // always @ (posedge CLK)

   always @(*)
     begin
        next_counter = counter;
        next_state = state;
        next_shift_reg = shift_reg;
        case(state)
          WAIT_FOR_RISING_LRCK:
            begin
               if(rising_lrck)
                 begin
                    next_state = WRITING_LEFT;
                    next_counter = 0; // this should coincide with the falling bick
                    next_shift_reg = DATA;
                 end
            end
          WRITING_LEFT:
            begin
               if(falling_bick)
                 begin
                    next_counter = counter + 1;
                    next_shift_reg = {shift_reg[1:47], 1'b0};
                    if(counter == 23)
                      next_state = WAIT_FOR_FALLING_LRCK;
                 end
            end
          WAIT_FOR_FALLING_LRCK:
            begin
               if(falling_lrck)
                 begin
                    next_counter = 0;
                    next_state = WRITING_RIGHT;
                 end
            end
          WRITING_RIGHT:
            begin
               if(falling_bick)
                 begin
                    next_counter = counter + 1;
                    next_shift_reg = {shift_reg[1:47], 1'b0};
                    if(counter == 23)
                      next_state = WAIT_FOR_RISING_LRCK;
                 end
            end
        endcase // case(state)
     end // always @ (*)
endmodule // codec_receiver

     